# region [Imports]

# * Standard Library Imports ---------------------------------------------------------------------------->
import os
import logging
from enum import Enum, auto
from typing import List, Union
# * Gid Imports ----------------------------------------------------------------------------------------->
import gidlogger as glog


# * Local Imports --------------------------------------------------------------------------------------->
from antistasi_console_log_parser.utils.gidsql.phrasers import GidSqliteInserter
from antistasi_console_log_parser.utils.gidsql.db_reader import Fetch, GidSqliteReader
from antistasi_console_log_parser.utils.gidsql.db_writer import GidSQLiteWriter
from antistasi_console_log_parser.utils.gidsql.script_handling import GidSqliteScriptProvider
# endregion[Imports]

__updated__ = '2020-11-28 03:29:05'

# region [AppUserData]

# endregion [AppUserData]

# region [Logging]

log = logging.getLogger('gidsql')


# endregion[Logging]

# region [Constants]
THIS_FILE_DIR = os.path.abspath(os.path.dirname(__file__))
# endregion[Constants]


class PhraseType(Enum):
    Insert = auto()
    Query = auto()
    Create = auto()
    Drop = auto()


class GidSqliteDatabase:
    Insert = PhraseType.Insert
    Query = PhraseType.Query
    Create = PhraseType.Create
    Drop = PhraseType.Drop

    All = Fetch.All
    One = Fetch.One

    phrase_objects = {Insert: GidSqliteInserter, Query: None, Create: None, Drop: None}

    def __init__(self, db_location, script_location, config=None, log_execution: bool = True):
        self.path = db_location
        self.script_location = script_location
        self.config = config
        self.pragmas = None
        if self.config is not None:
            self.pragmas = self.config.retrieve('general_settings', 'pragmas', typus=List[str], default_fallback=[])

        self.writer = GidSQLiteWriter(self.path, self.pragmas, log_execution=log_execution)
        self.reader = GidSqliteReader(self.path, self.pragmas, log_execution=log_execution)
        self.scripter = GidSqliteScriptProvider(self.script_location)

    def startup_db(self, overwrite=False):
        if os.path.exists(self.path) is True and overwrite is True:
            os.remove(self.path)
        for script in self.scripter.setup_scripts:
            self.writer.write(script)
        return True

    def new_phrase(self, typus: PhraseType):
        return self.phrase_objects.get(typus)()

    def write(self, phrase, variables=None, progress_handler: tuple = None):

        if isinstance(phrase, str):
            sql_phrase = self.scripter.get(phrase, None)
            if sql_phrase is None:
                sql_phrase = phrase
            self.writer.write(sql_phrase, variables, progress_handler=progress_handler)

    def query(self, phrase, variables=None, fetch: Fetch = Fetch.All, row_factory: Union[bool, any] = False):

        if row_factory:
            _factory = None if isinstance(row_factory, bool) is True else row_factory
            self.reader.enable_row_factory(in_factory=_factory)
        sql_phrase = self.scripter.get(phrase, None)
        if sql_phrase is None:
            sql_phrase = phrase
        _out = self.reader.query(sql_phrase, variables=variables, fetch=fetch)
        self.reader.disable_row_factory()
        return _out

    def vacuum(self):
        self.write('VACUUM')

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}({self.path}, {self.script_location}, {self.config})"

    def __str__(self) -> str:
        return self.__class__.__name__
